<?php

namespace Hej\Cti;

use SoapError;
use SugarWebServiceImplv4_1;

if (!defined('sugarEntry')) {
    define('sugarEntry', true);
}

require_once "service/v4_1/SugarWebServiceImplv4_1.php";
require_once "custom/Hej/CTI/HejSalesCti.php";

const MODULE_ID = "HejSoftwareCTI";

class HejSalesApi extends SugarWebServiceImplv4_1
{
    public function hejFetchSystemInfo()
    {
        global $suitecrm_version, $db;

        if (!$suitecrm_version) {
            require __DIR__ . "/../../../../suitecrm_version.php";
        }

        $installed = false;
        $enabled = null;
        $version = null;

        $res = $db->pQuery("SELECT version, status, enabled FROM upgrade_history WHERE status = 'installed' AND id_name = '?'", [MODULE_ID]);
        if ($res && $row = $db->fetchByAssoc($res)) {
            $version = $row["version"];
            $installed = $row["status"] === "installed";
            $enabled = $row["enabled"] === "1";
        }

        header("Content-Type: application/json");

        return [
            "version" => $suitecrm_version,
            "module" => compact("installed", "version", "enabled"),
        ];
    }

    /**
     * Search in contacts and leads by name
     *
     * @param string $session
     * @param string $name
     * @return array|bool
     */
    public function hejSearchContactsByName($session, $name)
    {
        if (!$this->validateSession($session)) {
            return false;
        }

        return [
            "contacts" => HejSalesCti::findContactByName($name),
        ];
    }

    /**
     * Search in contacts and leads by phone number
     *
     * @param string $session
     * @param string $phone
     * @return array|bool
     */
    public function hejSearchContactsByPhone($session, $phone)
    {
        if (!$this->validateSession($session)) {
            return false;
        }

        return [
            "contacts" => HejSalesCti::findContactByPhone($phone),
        ];
    }

    /**
     * Search in contacts and leads by email address
     *
     * @param string $session
     * @param string $email
     * @return array|bool
     */
    public function hejSearchContactsByEmail($session, $email)
    {
        if (!$this->validateSession($session)) {
            return false;
        }

        return [
            "contacts" => HejSalesCti::findContactByEmail($email),
        ];
    }

    /**
     * Ensure the session is valid
     *
     * @param string $session
     * @return bool
     */
    protected function validateSession($session)
    {
        return $this->checkReadAccess($session, "Contacts")
            && $this->checkReadAccess($session, "Leads");
    }

    /**
     * Ensure we have a valid session and access to the given module
     *
     * @param string $session
     * @param string $module
     * @return boolean
     */
    protected function checkReadAccess($session, $module)
    {
        $error = new SoapError();

        return self::$helperObject->checkSessionAndModuleAccess(
            $session,
            "invalid_session",
            $module,
            "read",
            "no_access",
            $error
        );
    }
}
